# coding: utf8
# Debian Contributors data source data mining tools
#
# Copyright (C) 2013  Enrico Zini <enrico@debian.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals
from .submission import Submission
from . import scanner
from ConfigParser import RawConfigParser

__all__ = ["Fail", "DataMine"]

class Fail(BaseException):
    pass

def read_config(fname):
    cp = RawConfigParser()
    cp.read(fname)
    for sec in cp.sections():
        yield sec, dict(cp.items(sec))

def read_scanners():
    import inspect
    for name, obj in inspect.getmembers(scanner):
        if not inspect.isclass(obj): continue
        if not hasattr(obj, "scan"): continue
        yield name.lower(), obj

class DataMine(object):
    def __init__(self, name, configfname):
        self.submission = Submission(name)
        self.scanners = []
        scanners = dict(read_scanners())
        for ctype, cfg in read_config(configfname):
            stype = cfg.get("type", None)
            if stype is None:
                raise Fail("scan configuration for {} has no scanner type".format(ctype))
            scanner = scanners.get(stype, None)
            if scanner is None:
                raise Fail("scan configuration for {} has unsupported type {}".format(ctype, stype))
            self.scanners.append(scanner(ctype, cfg))

    def scan(self):
        """
        Run all data miners and send all data to the given submission
        """
        for scanner in self.scanners:
            scanner.scan(self.submission)
