# coding: utf8
# Debian Contributors data source data mining tools for dak
#
# Copyright (C) 2013  Enrico Zini <enrico@debian.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals
from ..types import *
import email.utils
import psycopg2
import logging

log = logging.getLogger(__name__)

__all__ = ["DakUploaders", "DakMaintainers"]

class Dak(object):
    def __init__(self, ctype, cfg):
        self.db = psycopg2.connect(cfg["db"])
        self.ctype = ctype

    def query_uploaders(self):
        log.debug("Querying uploaders for %s...", self.ctype)
        cur = self.db.cursor()
        cur.execute("""
        SELECT s.install_date, u.uid, u.name
          FROM source s
          JOIN fingerprint f ON s.sig_fpr = f.id
          JOIN uid u ON f.uid = u.id
        """)
        for dt, uid, name in cur:
            if name is not None:
                name = name.decode("utf8", errors="replace")
            yield Identifier("login", uid, name), dt.date()

    def query_maintainers(self):
        log.debug("Querying maintainers for %s...", self.ctype)
        cur = self.db.cursor()
        cur.execute("""
        SELECT s.install_date, c.name
          FROM source s
          JOIN maintainer c ON s.changedby = c.id
        """)
        for dt, m_name in cur:
            realname, emailaddr = email.utils.parseaddr(m_name)
            realname = realname.decode("utf8", errors="replace")
            yield Identifier("email", emailaddr, realname), dt.date()

    def _query_to_submission(self, generator, submission):
        count_rows = 0
        by_ident = {}
        for ident, date in generator:
            count_rows += 1
            c = by_ident.get(ident, None)
            if c is None:
                by_ident[ident] = Contribution(self.ctype, date, date)
            else:
                c.extend_by_date(date)
        count_contribs = 0
        for ident, contrib in by_ident.iteritems():
            count_contribs += 1
            submission.add_contribution(ident, contrib)
        log.debug("%d rows read into %d contributions", count_rows, count_contribs)

class DakUploaders(Dak):
    """
    Scan git directories using file attributes to detect contributions
    """
    def scan(self, submission):
        self._query_to_submission(self.query_uploaders(), submission)


class DakMaintainers(Dak):
    """
    Scan git directories using file attributes to detect contributions
    """
    def scan(self, submission):
        self._query_to_submission(self.query_maintainers(), submission)

